package SlackBuild::Archive;
use strict;
use warnings;
use parent 'SlackBuild::URI';
use SlackBuild::Archive::Extractor;
use File::Temp qw/ tempfile tempdir /;
use File::Basename;
use File::Spec;
use Carp;

*AUTOLOAD = \&SlackBuild::URI::AUTOLOAD;

sub new {
    my ($class, $pname) = (shift, shift);
    my $self = $class->SUPER::new(@_);
    $self->{_package_name} = $pname;
    return $self;
}

sub package_name { shift->{_package_name} }
sub dir {
    my ($self, $n) = @_;
    if (defined($n)) {
	return undef unless $n <= @{$self->{_dir}};
	return $self->{_dir}[$n];
    }
    return @{$self->{_dir}}
}

sub iterate {
    my ($self, $cb) = @_;
    for (my $i = 0; my $file = $self->dir($i); $i++) {
	return 0 unless &{$cb}($file);
    }
    return 1;
}

sub add_file {
    my $self = shift;
    push @{$self->{_dir}}, File::Spec->no_upwards(map { s{/$}{}; $_ } @_);
}

sub has_file {
    my ($self, $file) = @_;
    return grep { $_ eq $file } ($self->dir);
}

sub verify {
    my $self = shift;
    my $file = $self->package_name . '.SlackBuild';
    return $self->error("no $file in archive")
	unless $self->has_file($file);
    # return $self->error("no slack-desc in archive")
    # 	unless $self->has_file('slack-desc');
    return 1;
}

sub download {
    croak "bad number of arguments" unless @_ == 2;
    my ($self, $dst) = @_;
    my (undef, $tmp) = tempfile(DIR => dirname($dst), UNLINK => 1);
    unless ($self->SUPER::download($tmp)) {
	return $self->error($self->SUPER::download_status);
    }
    unless (SlackBuild::Archive::Extractor->backend($self, $tmp, $dst)
	                                  ->extract) {
	return 0;
    }
    $self->_read_info($dst);
    return 1;
}

sub _read_info {
    my ($self, $dst) = @_;
    my $info = $self->package_name . '.info';
    return unless $self->has_file($info);
    open(my $fd, '<', File::Spec->catfile($dst, $info))
	or return;
    my %info;
    while (<$fd>) {
	chomp;
	s/^\s+//;
	next if /^(#.*)?$/;
	return unless /^(?<kw>[A-Za-z_][A-Za-z_0-9]*)="(?<val>.*)"$/;
	$info{$+{kw}} = $+{val};
    }
    close $fd;
    $self->{_info} = \%info;
}

sub info {
    my ($self, $kw) = @_;
    return undef unless exists $self->{_info};
    return $self->{_info}{$kw};
}

sub download_success {
    my $self = shift;
    return $self->{_last_success};
}

sub download_status {
    my $self = shift;
    return $self->SUPER::download_status unless exists $self->{_last_status};
    return $self->{_last_status};
}

sub error {
    my $self = shift;
    $self->{_last_success} = 0;
    $self->{_last_status} = shift;
    return 0;
}

1;
