package SlackBuild::Archive::Extractor::Tar;
use strict;
use warnings;
use parent 'SlackBuild::Archive::Extractor';
use POSIX::Run::Capture qw(:all);
use POSIX qw(:sys_wait_h);

sub _runcap_diag {
    my ($self, $obj) = @_;
    if (WIFEXITED($obj->status)) {
	return 1 if $obj->status == 0;
	while (my $s = $obj->next_line(SD_STDERR)) {
	    chomp($s);
	    $self->error($s);
	}
	$self->error("program "
		     . $obj->program
		     . " terminated with code "
		     . WEXITSTATUS($obj->status));
    } elsif (WIFSIGNALED($obj->status)) {
	$self->error("program "
		     . $obj->program
		     . " terminated on signal "
		     . WTERMSIG($obj->status));
    } else {
	$self->error("program "
		     . $obj->program
		     . " terminated with unrecogized code "
		     . $obj->status);
    }
    return 0;
}

sub _run_tar {
    my $self = shift;
    my $obj = new POSIX::Run::Capture(argv => [ 'tar', @_ ],
				      timeout => 10);
    if ($obj->run) {
	return undef unless $self->_runcap_diag($obj);
    } else {
	$self->error("can't run tar: ".strerror($obj->errno));
	return undef;
    }
    return $obj;
}

sub _check_build {
    my $self = shift;
    my $obj = $self->_run_tar('-t', '-f', $self->tempfile) or return 0;
    my $name = $self->archive->package_name;
    my $rx = qr($name);
    my $prefix;

    while (my $s = $obj->next_line(SD_STDOUT)) {
	chomp($s);
	$s =~ s{^\./}{};
	if ($s =~ m{^\./}) {
	    $self->error($self->slackbuild_uri .": bad file: $s");
	}
	if ($s =~ s{^$rx/}{}) {
	    if (defined($prefix)) {
		if (!$prefix) {
		    $self->error($self->slackbuild_uri .": bad file: $s");
		}
	    } else {
		$prefix = 1;
	    }
	} else {
	    if (defined($prefix)) {
		if ($prefix) {
		    $self->error($self->slackbuild_uri .": bad file: $s");
		}
	    } else {
		$prefix = 0;
	    }
	}
	$self->archive->add_file($s);
    }
    $self->{_skip} = $prefix;
    return $self->success;
}

sub extract {
    my $self = shift;
    return 0 unless $self->_check_build && $self->archive->verify;
    return $self->_run_tar('-C', $self->destdir,
			   '--strip', $self->{_skip},
			   '-x',
			   '-f',
			   $self->tempfile);
}

1;
