package SlackBuild::Registry::Backend::FS;
use strict;
use warnings;
use Carp;
use File::Basename;
use List::Regexp;
use File::stat;
use File::Spec;
use Fcntl ':mode';
use SlackBuild::Registry::Record;

=head1 NAME
    
SlackBuild::Registry::Backend::FS - filesystem backend for slackbuild registry

=head1 SYNOPSIS

    $reg = new SlackBuild::Registry(dir => DIRECTORY);
    my @a = $x->lookup('openssl', version => '1.0.2m');

=head1 METHODS

=head2 new

    $x = new SlackBuild::Registry::Backend::FS(dir => DIRECTORY)
    
=cut
    
sub new {
    my $class = shift;
    my $self = bless {}, $class;
    local %_ = @_;
    if (my $dir = delete $_{dir}) {
	$self->{dir} = $dir;
    } else {
	croak "required parameter dir not present";
    }
    croak "too many arguments" if keys %_;
    return $self;
}

my @architectures = qw(i386 x86_64 arm);
my @suffixes = qw(.tgz .txz);

=head2 lookup

    @a = $backend->lookup(PACKAGE, [version=>X], [arch=>Y], [build=>Z])

Returns a sorted array of SlackBuild::Registry::Record objects matching the
search criteria.    
    
=cut
    
sub lookup {
    my ($self, $pkg, %cond) = @_;
    my $v;

    my $pat = "$pkg-*-*-*";
    
    my $rx = '^' . qr($pkg) . '-'
	   . '(?<version>\d+(\.\d+)+.*?)-(?<arch>'
	   . regexp_opt(@architectures)
	   .= ')-(?<build>\d+)(?<rest>.*)$';

    my @result = sort {
	my $d;
	if ($d = ($a->package || '') cmp ($b->package || '')) {
	    $d
	} elsif ($d = $b->version <=> $a->version) {
	    $d
	} elsif ($a->arch && $b->arch
		 && ($d = $a->arch cmp $b->arch)) {
	    $d
        } else {
	    ($b->build || 1) <=> ($a->build || 1)
        }
    }
    grep { $_->matches(%cond) }
    map {
	my ($name,$path,$suffix) = fileparse($_, @suffixes);
	if ($name =~ m{$rx}) {
	    my $st = stat($_);
	    if (S_ISREG($st->mode)) {
		new SlackBuild::Registry::Record($pkg,
						 version => $+{version},
						 arch => $+{arch},
						 build => $+{build},
						 date => $st->mtime,
						 filename => File::Spec->abs2rel($_, $self->{dir}))
	    } else {
		()
	    }
	} else {
	    ()
	}
    } (glob File::Spec->catfile($self->{dir}, $pat));
    if (wantarray) {
	(@result)
    } else {
	shift @result;
    }
}

1;
